@@ -13,13 +13,14 @@
 
 #include <QByteArray>
 #include <QDataStream>
+#include <QIODevice>
 #include <QString>
-#include <QTextCodec>
-#include <QTextDecoder>
+#include <QStringDecoder>
 
 class EncodedDataStream: public QDataStream
 {
   private:
+    QStringEncoder mEncoder = QStringEncoder("UTF-8", QStringEncoder::Flag::ConvertInitialBom);
     QByteArray mEncoding = "UTF-8";
     bool mGenerateBOM = false;
     bool mError = false;
@@ -29,10 +30,12 @@ class EncodedDataStream: public QDataStream
 
     void setGenerateByteOrderMark(bool generate) { mGenerateBOM = generate; }
 
-    //Let the compiler choose the optimal solution based on c++ rules.
-    inline void setEncoding(const QByteArray &inEncoding) noexcept
+    inline bool hasBOM() const { return mGenerateBOM; }
+
+    inline void setEncoding(const QByteArray &inEncoding) 
     {
         assert(!inEncoding.isEmpty());
+
         if(inEncoding == "UTF-8-BOM")
         {
             mGenerateBOM = true;
@@ -43,61 +46,68 @@ class EncodedDataStream: public QDataStream
             mGenerateBOM = inEncoding.startsWith("UTF-16") || inEncoding.startsWith("UTF-32");
             mEncoding = inEncoding;
         }
-    };
 
-    inline void setEncoding(const QByteArray &&inEncoding) noexcept
-    {
-        assert(!inEncoding.isEmpty());
-        if(inEncoding == "UTF-8-BOM")
-        {
-            mGenerateBOM = true;
-            mEncoding = "UTF-8";
-        }
-        else
-        {
-            mGenerateBOM = inEncoding.startsWith("UTF") && !inEncoding.endsWith("-8");
-            mEncoding = inEncoding;
-        }
+        mEncoder = QStringEncoder(mEncoding, mGenerateBOM ? QStringConverter::Flag::WriteBom : QStringConverter::Flag::ConvertInitialBom);
+
+        assert(!mGenerateBOM || ((inEncoding.startsWith("UTF-16") || inEncoding.startsWith("UTF-32")) || inEncoding == "UTF-8-BOM"));
     };
 
     inline qint32 readChar(QChar& c)
     {
-        char curByte;
+        char curData = '\0';
         qint32 len = 0;
         QString s;
+        QStringDecoder decoder = QStringDecoder(mEncoding, mGenerateBOM ? QStringConverter::Flag::WriteBom : QStringConverter::Flag::ConvertInitialBom);
+        assert(decoder.isValid());
+        if(!decoder.isValid()) return 0;
 
-#if(QT_VERSION < QT_VERSION_CHECK(6, 0, 0))
-        QTextDecoder decoder = QTextDecoder(QTextCodec::codecForName(mEncoding), mGenerateBOM ? QTextCodec::ConversionFlag::DefaultConversion : QTextCodec::ConversionFlag::IgnoreHeader);
-#else
-        QTextDecoder decoder = QTextDecoder(QTextCodec::codecForName(mEncoding), mGenerateBOM ? QStringConverter::Flag::WriteBom : QStringConverter::Flag::ConvertInitialBom);
-#endif
-
+        /*
+            As long as the encoding is incomplete QStringDecoder returns an empty string and waits for more data.
+            This loop proceeds one byte at a time until we hit the end of input, get an encoded char and encounter an error.
+        */
         do
         {
-            len += readRawData(&curByte, 1);
-            s = decoder.toUnicode(&curByte, 1);
-        } while(!decoder.hasFailure() && decoder.needsMoreData() && !atEnd());
+            len += readRawData(&curData, 1);
 
-        mError = decoder.hasFailure();
+            s = decoder(QByteArray::fromRawData(&curData, sizeof(curData)));
+        } while(!decoder.hasError() && s.isEmpty() && !atEnd());
+
+        mError = decoder.hasError() || (s.isEmpty() && atEnd());
         if(!mError)
             c = s[0];
         else
-            c = QChar::ReplacementCharacter;
+            c = QChar::SpecialCharacter::ReplacementCharacter;
 
         return len;
     }
 
+    quint64 peekChar(QChar &c)
+    {
+        QStringDecoder decoder = QStringDecoder(mEncoding);
+        QIODevice *d = device();
+        char buf[4];
+        qint64 len = d->peek(buf, sizeof(buf));
+
+        if(len > 0)
+        {
+            QString s = decoder(QByteArray::fromRawData(buf, sizeof(buf)));
+            if(s.isEmpty()) return 0;
+
+            c = s[0];
+        }
+        else
+            c = QChar::Null;
+        return len;
+    }
+
     EncodedDataStream &operator<<(const QString &s)
     {
-#if(QT_VERSION < QT_VERSION_CHECK(6, 0, 0))
-        QTextEncoder encoder(QTextCodec::codecForName(mEncoding), mGenerateBOM ? QTextCodec::ConversionFlag::DefaultConversion : QTextCodec::ConversionFlag::IgnoreHeader);
-#else
-        QTextEncoder encoder(QTextCodec::codecForName(mEncoding), mGenerateBOM ? QStringConverter::Flag::WriteBom : QStringConverter::Flag::ConvertInitialBom);
-#endif
-        QByteArray data = encoder.fromUnicode(s);
-        mError = encoder.hasFailure();
+        QByteArray data = mEncoder(s);
 
-        return *this << data;
+        assert(mEncoder.isValid());
+        mError = mEncoder.hasError();
+        writeRawData(data.constData(), data.length());
+        return *this;
     };
 
     inline bool hasError() noexcept { return mError; }
