@@ -39,7 +39,6 @@
 #include <QFile>
 #include <QLayout>
 #include <QLineEdit>
-#include <QMimeData>
 #include <QPointer>
 #include <QProcess>
 #include <QScrollBar>
@@ -52,6 +51,26 @@
 #include <KMessageBox>
 #include <KShortcutsDialog>
 
+/**
+ * Clear all data that is specific to the current diff.
+ *
+ * This function is called when the application is reset, e.g. when a new diff is started.
+ */
+void KDiff3App::resetDiffData()
+{
+    assert(m_pDiffTextWindow1 != nullptr && m_pDiffTextWindow2 != nullptr && m_pDiffTextWindow3 != nullptr &&
+           m_pMergeResultWindow != nullptr);
+    //insure merge result window never has stale iterators/pointers.
+    m_pMergeResultWindow->reset();
+
+    m_diffList12.clear();
+    m_diffList23.clear();
+    m_diffList13.clear();
+    m_diff3LineList.clear();
+    mDiff3LineVector.clear();
+    m_manualDiffHelpList.clear();
+}
+
 void KDiff3App::mainInit(TotalDiffStatus* pTotalDiffStatus, const InitFlags inFlags)
 {
     ProgressScope pp;
@@ -102,13 +121,9 @@ void KDiff3App::mainInit(TotalDiffStatus* pTotalDiffStatus, const InitFlags inFl
         setLockPainting(true);
     }
 
-    m_diff3LineList.clear();
-    mDiff3LineVector.clear();
-
     if(bLoadFiles)
     {
-        m_manualDiffHelpList.clear();
-
+        resetDiffData();
         if(m_sd3->isEmpty())
             ProgressProxy::setMaxNofSteps(4); // Read 2 files, 1 comparison, 1 finediff
         else
@@ -139,6 +154,9 @@ void KDiff3App::mainInit(TotalDiffStatus* pTotalDiffStatus, const InitFlags inFl
     }
     else
     {
+        m_diff3LineList.clear();
+        mDiff3LineVector.clear();
+
         if(m_sd3->isEmpty())
             ProgressProxy::setMaxNofSteps(2); // 1 comparison, 1 finediff
         else
@@ -297,9 +315,7 @@ void KDiff3App::mainInit(TotalDiffStatus* pTotalDiffStatus, const InitFlags inFl
         }
         catch(const std::bad_alloc&)
         {
-            m_manualDiffHelpList.clear();
-            m_diff3LineList.clear();
-            mDiff3LineVector.clear();
+            resetDiffData();
             m_sd1->reset();
             m_sd2->reset();
             m_sd3->reset();
@@ -340,8 +356,6 @@ void KDiff3App::mainInit(TotalDiffStatus* pTotalDiffStatus, const InitFlags inFl
         mErrors.append(i18n("Too many lines in diff. Skipping file."));
     }
 
-    m_pMergeResultWindow->connectActions();
-
     m_pMainWidget->setVisible(bGUI); //sets off multiple resize events internally.
 
     m_bTripleDiff = !m_sd3->isEmpty();
@@ -421,7 +435,7 @@ void KDiff3App::createCaption()
     QString f1 = m_sd1->getAliasName();
     QString f2 = m_sd2->getAliasName();
     QString f3 = m_sd3->getAliasName();
-    QtSizeType p;
+    qsizetype p;
 
     if((p = f1.lastIndexOf('/')) >= 0 || (p = f1.lastIndexOf('\\')) >= 0)
         f1 = f1.mid(p + 1);
@@ -538,153 +552,6 @@ void KDiff3App::sourceMask(qint32 srcMask, qint32 enabledMask)
     chooseC->setEnabled((enabledMask & 4) != 0);
 }
 
-void KDiff3App::initView()
-{
-    // set the main widget here
-    if(mInitCalled)
-    {
-        return;
-    }
-
-    mInitCalled = true;
-    //m_pMainWidget // Contains vertical splitter and horiz scrollbar
-    QVBoxLayout* pVLayout = new QVBoxLayout(m_pMainWidget);
-    pVLayout->setContentsMargins(0, 0, 0, 0);
-    pVLayout->setSpacing(0);
-
-    QSplitter* pVSplitter = new QSplitter();
-    pVSplitter->setObjectName("VSplitter");
-    pVSplitter->setOpaqueResize(false);
-    pVSplitter->setOrientation(Qt::Vertical);
-    pVLayout->addWidget(pVSplitter);
-
-    QWidget* pDiffWindowFrame = new QWidget(); // Contains diff windows, overview and vert scrollbar
-    pDiffWindowFrame->setObjectName("DiffWindowFrame");
-    QHBoxLayout* pDiffHLayout = new QHBoxLayout(pDiffWindowFrame);
-    pDiffHLayout->setContentsMargins(0, 0, 0, 0);
-    pDiffHLayout->setSpacing(0);
-    pVSplitter->addWidget(pDiffWindowFrame);
-
-    m_pDiffWindowSplitter = new QSplitter();
-    m_pDiffWindowSplitter->setObjectName("DiffWindowSplitter");
-    m_pDiffWindowSplitter->setOpaqueResize(false);
-
-    m_pDiffWindowSplitter->setOrientation(gOptions->m_bHorizDiffWindowSplitting ? Qt::Horizontal : Qt::Vertical);
-    pDiffHLayout->addWidget(m_pDiffWindowSplitter);
-
-    m_pOverview = new Overview();
-    m_pOverview->setObjectName("Overview");
-    pDiffHLayout->addWidget(m_pOverview);
-
-    DiffTextWindow::mVScrollBar = new QScrollBar(Qt::Vertical, pDiffWindowFrame);
-    pDiffHLayout->addWidget(DiffTextWindow::mVScrollBar);
-
-    chk_connect_a(m_pOverview, &Overview::setLine, DiffTextWindow::mVScrollBar, &QScrollBar::setValue);
-    chk_connect_a(this, &KDiff3App::showWhiteSpaceToggled, m_pOverview, &Overview::slotRedraw);
-    chk_connect_a(this, &KDiff3App::changeOverViewMode, m_pOverview, &Overview::setOverviewMode);
-
-    m_pDiffTextWindowFrame1 = new DiffTextWindowFrame(m_pDiffWindowSplitter, e_SrcSelector::A, m_sd1, *this);
-    m_pDiffWindowSplitter->addWidget(m_pDiffTextWindowFrame1);
-    m_pDiffTextWindowFrame2 = new DiffTextWindowFrame(m_pDiffWindowSplitter, e_SrcSelector::B, m_sd2, *this);
-    m_pDiffWindowSplitter->addWidget(m_pDiffTextWindowFrame2);
-    m_pDiffTextWindowFrame3 = new DiffTextWindowFrame(m_pDiffWindowSplitter, e_SrcSelector::C, m_sd3, *this);
-    m_pDiffWindowSplitter->addWidget(m_pDiffTextWindowFrame3);
-    m_pDiffTextWindow1 = m_pDiffTextWindowFrame1->getDiffTextWindow();
-    m_pDiffTextWindow2 = m_pDiffTextWindowFrame2->getDiffTextWindow();
-    m_pDiffTextWindow3 = m_pDiffTextWindowFrame3->getDiffTextWindow();
-
-    m_pDiffTextWindowFrame1->setupConnections(this);
-    m_pDiffTextWindowFrame2->setupConnections(this);
-    m_pDiffTextWindowFrame3->setupConnections(this);
-
-    // Merge window
-    m_pMergeWindowFrame = new QWidget(pVSplitter);
-    m_pMergeWindowFrame->setObjectName("MergeWindowFrame");
-    pVSplitter->addWidget(m_pMergeWindowFrame);
-    QHBoxLayout* pMergeHLayout = new QHBoxLayout(m_pMergeWindowFrame);
-    pMergeHLayout->setContentsMargins(0, 0, 0, 0);
-    pMergeHLayout->setSpacing(0);
-    QVBoxLayout* pMergeVLayout = new QVBoxLayout();
-    pMergeHLayout->addLayout(pMergeVLayout, 1);
-
-    m_pMergeResultWindowTitle = new WindowTitleWidget();
-    pMergeVLayout->addWidget(m_pMergeResultWindowTitle);
-
-    m_pMergeResultWindow = new MergeResultWindow(m_pMergeWindowFrame, statusBar());
-    pMergeVLayout->addWidget(m_pMergeResultWindow, 1);
-
-    MergeResultWindow::mVScrollBar = new QScrollBar(Qt::Vertical, m_pMergeWindowFrame);
-    pMergeHLayout->addWidget(MergeResultWindow::mVScrollBar);
-
-    autoAdvance->setEnabled(true);
-
-    QList<qint32> sizes = pVSplitter->sizes();
-    qint32 total = sizes[0] + sizes[1];
-    if(total < 10)
-        total = 100;
-    sizes[0] = total / 2;
-    sizes[1] = total / 2;
-    pVSplitter->setSizes(sizes);
-
-    QList<qint32> hSizes = {1, 1, 1};
-
-    m_pDiffWindowSplitter->setSizes(hSizes);
-
-    m_pMergeResultWindow->installEventFilter(m_pMergeResultWindowTitle); // for focus tracking
-
-    QHBoxLayout* pHScrollBarLayout = new QHBoxLayout();
-    pVLayout->addLayout(pHScrollBarLayout);
-    m_pHScrollBar = new ReversibleScrollBar(Qt::Horizontal, &gOptions->m_bRightToLeftLanguage);
-    pHScrollBarLayout->addWidget(m_pHScrollBar);
-    m_pCornerWidget = new QWidget(m_pMainWidget);
-    pHScrollBarLayout->addWidget(m_pCornerWidget);
-
-    chk_connect_a(DiffTextWindow::mVScrollBar, &QScrollBar::valueChanged, m_pOverview, &Overview::setFirstLine);
-    chk_connect_a(DiffTextWindow::mVScrollBar, &QScrollBar::valueChanged, m_pDiffTextWindow1, &DiffTextWindow::setFirstLine);
-    chk_connect_a(m_pHScrollBar, &ReversibleScrollBar::valueChanged2, m_pDiffTextWindow1, &DiffTextWindow::setHorizScrollOffset);
-    m_pDiffTextWindow1->setupConnections(this);
-
-    chk_connect_a(DiffTextWindow::mVScrollBar, &QScrollBar::valueChanged, m_pDiffTextWindow2, &DiffTextWindow::setFirstLine);
-    chk_connect_a(m_pHScrollBar, &ReversibleScrollBar::valueChanged2, m_pDiffTextWindow2, &DiffTextWindow::setHorizScrollOffset);
-    m_pDiffTextWindow2->setupConnections(this);
-
-    chk_connect_a(DiffTextWindow::mVScrollBar, &QScrollBar::valueChanged, m_pDiffTextWindow3, &DiffTextWindow::setFirstLine);
-    chk_connect_a(m_pHScrollBar, &ReversibleScrollBar::valueChanged2, m_pDiffTextWindow3, &DiffTextWindow::setHorizScrollOffset);
-    m_pDiffTextWindow3->setupConnections(this);
-
-    MergeResultWindow* p = m_pMergeResultWindow;
-    chk_connect_a(MergeResultWindow::mVScrollBar, &QScrollBar::valueChanged, p, &MergeResultWindow::setFirstLine);
-
-    chk_connect_a(m_pHScrollBar, &ReversibleScrollBar::valueChanged2, p, &MergeResultWindow::setHorizScrollOffset);
-    chk_connect_a(p, &MergeResultWindow::modifiedChanged, m_pMergeResultWindowTitle, &WindowTitleWidget::slotSetModified);
-    p->setupConnections(this);
-    sourceMask(0, 0);
-
-    chk_connect_a(p, &MergeResultWindow::setFastSelectorRange, m_pDiffTextWindow1, &DiffTextWindow::setFastSelectorRange);
-    chk_connect_a(p, &MergeResultWindow::setFastSelectorRange, m_pDiffTextWindow2, &DiffTextWindow::setFastSelectorRange);
-    chk_connect_a(p, &MergeResultWindow::setFastSelectorRange, m_pDiffTextWindow3, &DiffTextWindow::setFastSelectorRange);
-    chk_connect_a(m_pDiffTextWindow1, &DiffTextWindow::setFastSelectorLine, p, &MergeResultWindow::slotSetFastSelectorLine);
-    chk_connect_a(m_pDiffTextWindow2, &DiffTextWindow::setFastSelectorLine, p, &MergeResultWindow::slotSetFastSelectorLine);
-    chk_connect_a(m_pDiffTextWindow3, &DiffTextWindow::setFastSelectorLine, p, &MergeResultWindow::slotSetFastSelectorLine);
-    chk_connect_a(m_pDiffTextWindow1, &DiffTextWindow::gotFocus, p, &MergeResultWindow::updateSourceMask);
-    chk_connect_a(m_pDiffTextWindow2, &DiffTextWindow::gotFocus, p, &MergeResultWindow::updateSourceMask);
-    chk_connect_a(m_pDiffTextWindow3, &DiffTextWindow::gotFocus, p, &MergeResultWindow::updateSourceMask);
-    chk_connect_a(m_pDirectoryMergeInfo, &DirectoryMergeInfo::gotFocus, p, &MergeResultWindow::updateSourceMask);
-
-    chk_connect_a(m_pDiffTextWindow1, &DiffTextWindow::resizeHeightChangedSignal, this, &KDiff3App::resizeDiffTextWindowHeight);
-    // The following connects cause the wordwrap to be recalced thrice, just to make sure. Better than forgetting one.
-    chk_connect_a(m_pDiffTextWindow1, &DiffTextWindow::resizeWidthChangedSignal, this, &KDiff3App::postRecalcWordWrap);
-    chk_connect_a(m_pDiffTextWindow2, &DiffTextWindow::resizeWidthChangedSignal, this, &KDiff3App::postRecalcWordWrap);
-    chk_connect_a(m_pDiffTextWindow3, &DiffTextWindow::resizeWidthChangedSignal, this, &KDiff3App::postRecalcWordWrap);
-
-    m_pDiffTextWindow1->setFocus();
-    m_pMainWidget->setMinimumSize(50, 50);
-    m_pCornerWidget->setFixedSize(DiffTextWindow::mVScrollBar->width(), m_pHScrollBar->height());
-    showWindowA->setChecked(true);
-    showWindowB->setChecked(true);
-    showWindowC->setChecked(true);
-}
-
 // called after word wrap is complete
 void KDiff3App::slotFinishMainInit()
 {
@@ -1065,7 +932,7 @@ void KDiff3App::slotEditPaste()
 {
     slotStatusMsg(i18n("Inserting clipboard contents..."));
 
-    if(m_pMergeResultWindow != nullptr && m_pMergeResultWindow->isVisible())
+    if(m_pMergeResultWindow->isVisible())
     {
         m_pMergeResultWindow->pasteClipboard(false);
     }
@@ -1127,53 +994,18 @@ void KDiff3App::slotEditSelectAll()
     slotStatusMsg(i18n("Ready."));
 }
 
-void KDiff3App::slotGoCurrent()
-{
-    Q_EMIT goCurrent();
-}
-
-void KDiff3App::slotGoTop()
-{
-    Q_EMIT goTop();
-}
-
-void KDiff3App::slotGoBottom()
-{
-    Q_EMIT goBottom();
-}
-
-void KDiff3App::slotGoPrevUnsolvedConflict()
-{
-    Q_EMIT goPrevUnsolvedConflict();
-}
-
 void KDiff3App::slotGoNextUnsolvedConflict()
 {
     m_bTimerBlock = false;
     Q_EMIT goNextUnsolvedConflict();
 }
 
-void KDiff3App::slotGoPrevConflict()
-{
-    Q_EMIT goPrevConflict();
-}
-
 void KDiff3App::slotGoNextConflict()
 {
     m_bTimerBlock = false;
     Q_EMIT goNextConflict();
 }
 
-void KDiff3App::slotGoPrevDelta()
-{
-    Q_EMIT goPrevDelta();
-}
-
-void KDiff3App::slotGoNextDelta()
-{
-    Q_EMIT goNextDelta();
-}
-
 void KDiff3App::slotGoToLine()
 {
     QDialog pDialog;
@@ -1209,7 +1041,7 @@ void KDiff3App::choose(e_SrcSelector choice)
 {
     if(!m_bTimerBlock)
     {
-        if(m_pDirectoryMergeWindow && m_pDirectoryMergeWindow->hasFocus())
+        if(m_pDirectoryMergeWindow->hasFocus())
         {
             if(choice == e_SrcSelector::A) m_pDirectoryMergeWindow->slotCurrentChooseA();
             if(choice == e_SrcSelector::B) m_pDirectoryMergeWindow->slotCurrentChooseB();
@@ -1219,7 +1051,7 @@ void KDiff3App::choose(e_SrcSelector choice)
             chooseB->setChecked(false);
             chooseC->setChecked(false);
         }
-        else if(m_pMergeResultWindow)
+        else
         {
             m_pMergeResultWindow->choose(choice);
             if(autoAdvance->isChecked())
@@ -1231,19 +1063,6 @@ void KDiff3App::choose(e_SrcSelector choice)
     }
 }
 
-void KDiff3App::slotChooseA()
-{
-    choose(e_SrcSelector::A);
-}
-void KDiff3App::slotChooseB()
-{
-    choose(e_SrcSelector::B);
-}
-void KDiff3App::slotChooseC()
-{
-    choose(e_SrcSelector::C);
-}
-
 void KDiff3App::slotAutoSolve()
 {
     Q_EMIT autoSolve();
@@ -1251,42 +1070,24 @@ void KDiff3App::slotAutoSolve()
     Q_EMIT updateAvailabilities();
 }
 
-void KDiff3App::slotUnsolve()
-{
-    Q_EMIT unsolve();
-}
-
-void KDiff3App::slotMergeHistory()
-{
-    Q_EMIT mergeHistory();
-}
-
-void KDiff3App::slotRegExpAutoMerge()
-{
-    Q_EMIT regExpAutoMerge();
-}
-
 void KDiff3App::slotSplitDiff()
 {
     LineRef firstLine;
     LineRef lastLine;
     QPointer<DiffTextWindow> pDTW = nullptr;
-    if(m_pDiffTextWindow1)
-    {
-        pDTW = m_pDiffTextWindow1;
-        pDTW->getSelectionRange(&firstLine, &lastLine, eD3LLineCoords);
-    }
-    if(!firstLine.isValid() && m_pDiffTextWindow2)
+    pDTW->getSelectionRange(&firstLine, &lastLine, eD3LLineCoords);
+
+    if(!firstLine.isValid())
     {
         pDTW = m_pDiffTextWindow2;
         pDTW->getSelectionRange(&firstLine, &lastLine, eD3LLineCoords);
     }
-    if(!firstLine.isValid() && m_pDiffTextWindow3)
+    if(!firstLine.isValid())
     {
         pDTW = m_pDiffTextWindow3;
         pDTW->getSelectionRange(&firstLine, &lastLine, eD3LLineCoords);
     }
-    if(pDTW && firstLine.isValid() && m_pMergeResultWindow)
+    if(pDTW && firstLine.isValid())
     {
         pDTW->resetSelection();
 
@@ -1299,22 +1100,21 @@ void KDiff3App::slotJoinDiffs()
     LineRef firstLine;
     LineRef lastLine;
     QPointer<DiffTextWindow> pDTW = nullptr;
-    if(m_pDiffTextWindow1)
-    {
-        pDTW = m_pDiffTextWindow1;
-        pDTW->getSelectionRange(&firstLine, &lastLine, eD3LLineCoords);
-    }
-    if(!firstLine.isValid() && m_pDiffTextWindow2)
+
+    pDTW = m_pDiffTextWindow1;
+    pDTW->getSelectionRange(&firstLine, &lastLine, eD3LLineCoords);
+
+    if(!firstLine.isValid())
     {
         pDTW = m_pDiffTextWindow2;
         pDTW->getSelectionRange(&firstLine, &lastLine, eD3LLineCoords);
     }
-    if(!firstLine.isValid() && m_pDiffTextWindow3)
+    if(!firstLine.isValid())
     {
         pDTW = m_pDiffTextWindow3;
         pDTW->getSelectionRange(&firstLine, &lastLine, eD3LLineCoords);
     }
-    if(pDTW && firstLine.isValid() && m_pMergeResultWindow)
+    if(pDTW && firstLine.isValid())
     {
         pDTW->resetSelection();
 
@@ -1338,18 +1138,13 @@ void KDiff3App::slotConfigureKeys()
 
 void KDiff3App::slotRefresh()
 {
+    assert(m_pDiffWindowSplitter != nullptr && m_pHScrollBar != nullptr);
     QApplication::setFont(gOptions->appFont());
 
     Q_EMIT doRefresh();
 
-    if(m_pHScrollBar != nullptr)
-    {
-        m_pHScrollBar->setAgain();
-    }
-    if(m_pDiffWindowSplitter != nullptr)
-    {
-        m_pDiffWindowSplitter->setOrientation(gOptions->m_bHorizDiffWindowSplitting ? Qt::Horizontal : Qt::Vertical);
-    }
+    m_pHScrollBar->setAgain();
+    m_pDiffWindowSplitter->setOrientation(gOptions->m_bHorizDiffWindowSplitting ? Qt::Horizontal : Qt::Vertical);
 }
 
 void KDiff3App::slotSelectionStart()
@@ -1387,21 +1182,6 @@ void KDiff3App::slotSelectionEnd()
     Q_EMIT updateAvailabilities();
 }
 
-void KDiff3App::slotClipboardChanged()
-{
-    const QClipboard* clipboard = QApplication::clipboard();
-    const QMimeData* mimeData = clipboard->mimeData();
-    if(mimeData && mimeData->hasText())
-    {
-        QString s = clipboard->text();
-        editPaste->setEnabled(!s.isEmpty());
-    }
-    else
-    {
-        editPaste->setEnabled(false);
-    }
-}
-
 void KDiff3App::slotOutputModified(bool bModified)
 {
     if(bModified && !m_bOutputModified)
@@ -1444,7 +1224,12 @@ void KDiff3App::postRecalcWordWrap()
     }
     else
     {
-        g_pProgressDialog->cancel(ProgressDialog::eResize);
+        /*
+            Clear exiting wordwrap threads and prevent recal signals from completing.
+            This honors the intent of the old cancel call without the risk of aborting file I/O.
+        */
+        QSignalBlocker blocker(m_pDiffTextWindow1), blocker2(m_pDiffTextWindow2), blocker3(m_pDiffTextWindow3), blocker4(m_pMergeResultWindow);
+        while(DiffTextWindow::maxThreads() > 0) {} //Clear wordwrap threads.
     }
 }
 
@@ -1850,6 +1635,12 @@ void KDiff3App::slotEditFind()
     }
 }
 
+void KDiff3App::slotScrollToH(qsizetype p)
+{
+    QString s = m_pFindDialog->m_pSearchString->text();
+    m_pHScrollBar->setValue(std::max<SafeInt<qint32>>(0, p + s.length() - m_pHScrollBar->pageStep()));
+}
+
 void KDiff3App::slotEditFindNext()
 {
     QString s = m_pFindDialog->m_pSearchString->text();
@@ -1863,17 +1654,13 @@ void KDiff3App::slotEditFindNext()
     bool bCaseSensitive = m_pFindDialog->m_pCaseSensitive->isChecked();
 
     LineRef d3vLine = m_pFindDialog->currentLine;
-    QtSizeType posInLine = m_pFindDialog->currentPos;
-    LineRef l;
-    QtSizeType p = 0;
+    qsizetype posInLine = m_pFindDialog->currentPos;
+
     if(m_pFindDialog->getCurrentWindow() == eWindowIndex::A)
     {
         if(m_pFindDialog->m_pSearchInA->isChecked() && m_pDiffTextWindow1 != nullptr &&
            m_pDiffTextWindow1->findString(s, d3vLine, posInLine, bDirDown, bCaseSensitive))
         {
-            m_pDiffTextWindow1->setSelection(d3vLine, posInLine, d3vLine, posInLine + s.length(), l, p);
-            DiffTextWindow::mVScrollBar->setValue(l - DiffTextWindow::mVScrollBar->pageStep() / 2);
-            m_pHScrollBar->setValue(std::max<SafeInt<qint32>>(0, p + s.length() - m_pHScrollBar->pageStep()));
             m_pFindDialog->currentLine = d3vLine;
             m_pFindDialog->currentPos = posInLine + 1;
             return;
@@ -1888,9 +1675,6 @@ void KDiff3App::slotEditFindNext()
         if(m_pFindDialog->m_pSearchInB->isChecked() && m_pDiffTextWindow2 != nullptr &&
            m_pDiffTextWindow2->findString(s, d3vLine, posInLine, bDirDown, bCaseSensitive))
         {
-            m_pDiffTextWindow2->setSelection(d3vLine, posInLine, d3vLine, posInLine + s.length(), l, p);
-            DiffTextWindow::mVScrollBar->setValue(l - DiffTextWindow::mVScrollBar->pageStep() / 2);
-            m_pHScrollBar->setValue(std::max<SafeInt<qint32>>(0, p + s.length() - m_pHScrollBar->pageStep()));
             m_pFindDialog->currentLine = d3vLine;
             m_pFindDialog->currentPos = posInLine + 1;
             return;
@@ -1906,9 +1690,6 @@ void KDiff3App::slotEditFindNext()
         if(m_pFindDialog->m_pSearchInC->isChecked() && m_pDiffTextWindow3 != nullptr &&
            m_pDiffTextWindow3->findString(s, d3vLine, posInLine, bDirDown, bCaseSensitive))
         {
-            m_pDiffTextWindow3->setSelection(d3vLine, posInLine, d3vLine, posInLine + s.length(), l, p);
-            DiffTextWindow::mVScrollBar->setValue(l - DiffTextWindow::mVScrollBar->pageStep() / 2);
-            m_pHScrollBar->setValue(std::max<SafeInt<qint32>>(0, p + s.length() - m_pHScrollBar->pageStep()));
             m_pFindDialog->currentLine = d3vLine;
             m_pFindDialog->currentPos = posInLine + 1;
             return;
@@ -1924,9 +1705,6 @@ void KDiff3App::slotEditFindNext()
         if(m_pFindDialog->m_pSearchInOutput->isChecked() && m_pMergeResultWindow != nullptr && m_pMergeResultWindow->isVisible() &&
            m_pMergeResultWindow->findString(s, d3vLine, posInLine, bDirDown, bCaseSensitive))
         {
-            m_pMergeResultWindow->setSelection(d3vLine, posInLine, d3vLine, posInLine + s.length());
-            MergeResultWindow::mVScrollBar->setValue(d3vLine - MergeResultWindow::mVScrollBar->pageStep() / 2);
-            m_pHScrollBar->setValue(std::max<SafeInt<qint32>>(0, posInLine + s.length() - m_pHScrollBar->pageStep()));
             m_pFindDialog->currentLine = d3vLine;
             m_pFindDialog->currentPos = posInLine + 1;
             return;
@@ -2141,17 +1919,14 @@ void KDiff3App::slotEncodingChanged(const QByteArray&)
 
 void KDiff3App::slotUpdateAvailabilities()
 {
-    if(m_pDiffTextWindow2 == nullptr || m_pDiffTextWindow1 == nullptr || m_pDiffTextWindow3 == nullptr)
-        return;
+    assert(m_pDiffTextWindow2 != nullptr && m_pDiffTextWindow1 != nullptr && m_pDiffTextWindow3 != nullptr);
 
     bool bTextDataAvailable = (m_sd1->hasData() || m_sd2->hasData() || m_sd3->hasData());
 
     if(dirShowBoth->isChecked())
     {
-        if(m_pDirectoryMergeDock != nullptr)
-            m_pDirectoryMergeDock->setVisible(m_bDirCompare);
-        if(m_pDirectoryMergeInfoDock != nullptr)
-            m_pDirectoryMergeInfoDock->setVisible(m_bDirCompare);
+        m_pDirectoryMergeDock->setVisible(m_bDirCompare);
+        m_pDirectoryMergeInfoDock->setVisible(m_bDirCompare);
 
         if(!m_pMainWidget->isVisible() &&
            bTextDataAvailable && !m_pDirectoryMergeWindow->isScanning())
@@ -2159,26 +1934,21 @@ void KDiff3App::slotUpdateAvailabilities()
     }
 
     bool bDiffWindowVisible = m_pMainWidget->isVisible();
-    bool bMergeEditorVisible = m_pMergeWindowFrame != nullptr && m_pMergeWindowFrame->isVisible() && m_pMergeResultWindow != nullptr;
+    bool bMergeEditorVisible = m_pMergeWindowFrame->isVisible();
 
     m_pDirectoryMergeWindow->updateAvailabilities(bMergeEditorVisible, m_bDirCompare, bDiffWindowVisible, chooseA, chooseB, chooseC);
 
     dirShowBoth->setEnabled(m_bDirCompare);
     dirViewToggle->setEnabled(
         m_bDirCompare &&
-        (m_pDirectoryMergeDock != nullptr && m_pDirectoryMergeInfoDock != nullptr &&
-         ((!m_pDirectoryMergeDock->isVisible() && m_pMainWidget->isVisible()) ||
-          (m_pDirectoryMergeDock->isVisible() && !m_pMainWidget->isVisible() && bTextDataAvailable))));
+        ((!m_pDirectoryMergeDock->isVisible() && m_pMainWidget->isVisible()) ||
+         (m_pDirectoryMergeDock->isVisible() && !m_pMainWidget->isVisible() && bTextDataAvailable)));
 
     showWhiteSpaceCharacters->setEnabled(bDiffWindowVisible);
     autoAdvance->setEnabled(bMergeEditorVisible);
     mAutoSolve->setEnabled(bMergeEditorVisible && m_bTripleDiff);
     mUnsolve->setEnabled(bMergeEditorVisible);
 
-    editUndo->setEnabled(false); //Not yet implemented but planned.
-    editCut->setEnabled(allowCut());
-    editCopy->setEnabled(allowCopy());
-
     mMergeHistory->setEnabled(bMergeEditorVisible);
     mergeRegExp->setEnabled(bMergeEditorVisible);
     showWindowA->setEnabled(bDiffWindowVisible && (m_pDiffTextWindow2->isVisible() || m_pDiffTextWindow3->isVisible()));
@@ -2188,26 +1958,23 @@ void KDiff3App::slotUpdateAvailabilities()
     editFindNext->setEnabled(bDiffWindowVisible);
     m_pFindDialog->m_pSearchInC->setEnabled(m_bTripleDiff);
     m_pFindDialog->m_pSearchInOutput->setEnabled(bMergeEditorVisible);
+    stdMenus->updateAvailabilities();
 
-    bool bSavable = bMergeEditorVisible && m_pMergeResultWindow->getNumberOfUnsolvedConflicts() == 0;
-    fileSave->setEnabled(m_bOutputModified && bSavable);
-    fileSaveAs->setEnabled(bSavable);
-
-    mGoTop->setEnabled(bDiffWindowVisible && m_pMergeResultWindow != nullptr && m_pMergeResultWindow->isDeltaAboveCurrent());
-    mGoBottom->setEnabled(bDiffWindowVisible && m_pMergeResultWindow != nullptr && m_pMergeResultWindow->isDeltaBelowCurrent());
+    mGoTop->setEnabled(bDiffWindowVisible && m_pMergeResultWindow->isDeltaAboveCurrent());
+    mGoBottom->setEnabled(bDiffWindowVisible && m_pMergeResultWindow->isDeltaBelowCurrent());
     mGoCurrent->setEnabled(bDiffWindowVisible);
     mGoPrevUnsolvedConflict->setEnabled(bMergeEditorVisible && m_pMergeResultWindow->isUnsolvedConflictAboveCurrent());
     mGoNextUnsolvedConflict->setEnabled(bMergeEditorVisible && m_pMergeResultWindow->isUnsolvedConflictBelowCurrent());
     mGoPrevConflict->setEnabled(bDiffWindowVisible && bMergeEditorVisible && m_pMergeResultWindow->isConflictAboveCurrent());
     mGoNextConflict->setEnabled(bDiffWindowVisible && bMergeEditorVisible && m_pMergeResultWindow->isConflictBelowCurrent());
-    mGoPrevDelta->setEnabled(bDiffWindowVisible && m_pMergeResultWindow != nullptr && m_pMergeResultWindow->isDeltaAboveCurrent());
-    mGoNextDelta->setEnabled(bDiffWindowVisible && m_pMergeResultWindow != nullptr && m_pMergeResultWindow->isDeltaBelowCurrent());
+    mGoPrevDelta->setEnabled(bDiffWindowVisible && m_pMergeResultWindow->isDeltaAboveCurrent());
+    mGoNextDelta->setEnabled(bDiffWindowVisible && m_pMergeResultWindow->isDeltaBelowCurrent());
 
     overviewModeNormal->setEnabled(m_bTripleDiff && bDiffWindowVisible);
     overviewModeAB->setEnabled(m_bTripleDiff && bDiffWindowVisible);
     overviewModeAC->setEnabled(m_bTripleDiff && bDiffWindowVisible);
     overviewModeBC->setEnabled(m_bTripleDiff && bDiffWindowVisible);
-    e_OverviewMode overviewMode = m_pOverview == nullptr ? e_OverviewMode::eOMNormal : m_pOverview->getOverviewMode();
+    e_OverviewMode overviewMode = m_pOverview->getOverviewMode();
     overviewModeNormal->setChecked(overviewMode == e_OverviewMode::eOMNormal);
     overviewModeAB->setChecked(overviewMode == e_OverviewMode::eOMAvsB);
     overviewModeAC->setChecked(overviewMode == e_OverviewMode::eOMAvsC);
